package main

import androidx.compose.desktop.ui.tooling.preview.Preview
import androidx.compose.runtime.Composable
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.foundation.layout.*
import androidx.compose.material.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Check
import androidx.compose.material.icons.filled.Close
import java.awt.FileDialog
import java.awt.Frame
import androidx.compose.material.Button
import androidx.compose.material.MaterialTheme
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.window.*
import kotlinx.serialization.json.Json
import java.io.File
import java.io.FileWriter
import java.io.IOException
import kotlinx.serialization.encodeToString

val customRedColor = Color(0xFFB70D1B)

@Composable
fun homePage(
    onGlossaireClick: () -> Unit,
    onCodeAVerifierClick: () -> Unit
) {

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {
        Text("Quali'Nomme", style = MaterialTheme.typography.h3)

        Spacer(modifier = Modifier.height(16.dp))

        Row(
            horizontalArrangement = Arrangement.spacedBy(16.dp)
        ) {
            Button(
                onClick = onGlossaireClick,
                colors = ButtonDefaults.buttonColors(
                    backgroundColor = customRedColor,
                    contentColor = Color.White
                )
            ) {
                Text("Glossaire")
            }

            Button(onClick = onCodeAVerifierClick,
                colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            ) ) {
                Text("Code à Vérifier")
            }

            Button(onClick = { /* Action de Comparer */ },
                colors = ButtonDefaults.buttonColors(
                    backgroundColor = customRedColor,
                    contentColor = Color.White
                )
            ) {
                Text("Comparer")
            }
        }
    }
}

@Composable
@Preview
fun app() {
    MaterialTheme {
        val currentPage = remember { mutableStateOf("accueil") }

        when (currentPage.value) {
            "accueil" -> {
                homePage(
                    onGlossaireClick = { currentPage.value = "glossaire" },
                    onCodeAVerifierClick = { currentPage.value = "choixLangage" }
                )
            }
            "glossaire" -> {
                glossairePage(
                    onAjouterMotClick = { currentPage.value = "formulaire" },
                    onImporterClick = {
                        val fileDialog = FileDialog(Frame(), "Select a file", FileDialog.LOAD)
                        fileDialog.file = "Untitled.csv" // Initial file name
                        fileDialog.isMultipleMode = false // To enable selecting only one file

                        fileDialog.setFile("*.csv")
                        fileDialog.isVisible = true

                        val selectedFile = fileDialog.file
                        val selectedDirectory = fileDialog.directory

                        if (selectedFile != null) {
                            val filePath = selectedDirectory + selectedFile
                            println("Opening: $filePath")

                            importCSV(filePath)

                        } else {
                            println("Open command cancelled by user.")
                        }
                    },
                    onExporterClick = {
                        val fileDialog = FileDialog(Frame(), "Save as CSV", FileDialog.SAVE)
                        fileDialog.file = "glossaire_exporte.csv" // Initial file name
                        fileDialog.isVisible = true
                        val selectedFile = fileDialog.file
                        val selectedDirectory = fileDialog.directory
                        if (selectedFile != null) {
                            val csvFilePath = "$selectedDirectory$selectedFile"
                            println("Exporting to: $csvFilePath")
                            exportToCSV(csvFilePath)
                        } else {
                            println("Export command cancelled by user.")
                        }
                    },
                    onRetourClick = { currentPage.value = "accueil" }
                )
            }
            "formulaire" -> {
                formulairePage(onAnnulerClick = { currentPage.value = "glossaire" })
            }
            "choixLangage" -> {
                val pythonExtensions = setOf("py")
                val javaExtensions = setOf("java")
                val jsExtensions = setOf("js", "html")
                choixLangagePage(
                    onRetourClick = { currentPage.value = "accueil" },
                    onPythonClick = {
                        selectFile(pythonExtensions) { filePath ->
                            println("Python file selected: $filePath") // Change by parser functions
                        } },
                    onJavaClick = {
                        selectFile(javaExtensions) { filePath ->
                            println("Java file selected: $filePath") // Change by parser functions
                        }
                    },
                    onJavaScriptClick = {
                        selectFile(jsExtensions) { filePath ->
                            parser(filePath) // Change by parser functions
                        }
                    }
                )
            }
            // ... Ajoutez d'autres cas pour les pages "glossaire," "code a verifier," et "comparer"
        }
    }
}

fun selectFile(extensions: Set<String>, onFileSelected: (String) -> Unit) {
    val fileDialog = FileDialog(Frame(), "Select a file", FileDialog.LOAD)
    fileDialog.isMultipleMode = false // To enable selecting only one file
    fileDialog.file = "*." + extensions.joinToString(";*.")
    fileDialog.isVisible = true



    val selectedFile = fileDialog.file
    val selectedDirectory = fileDialog.directory

    if (selectedFile != null) {
        val filePath = "$selectedDirectory$selectedFile"

        // Vérifier si l'extension est autorisée
        val fileExtension = File(filePath).extension.lowercase()
        if (extensions.contains(fileExtension)) {
            println("Opening: $filePath")
            onFileSelected(filePath)
        } else {
            println("Invalid file extension.")
        }
    } else {
        println("Open command cancelled by user.")
    }
}

fun exportToCSV(csvFilePath: String) {
    val glossary = chargerDonneesDepuisFichier()
    val csvContent = buildString {
        appendLine("Mot;Description;Contexte principal;Contexte 2;Lié à;Synonyme;Antonyme;")
        glossary.forEach { entry ->
            appendLine(
                "${entry.nom};${entry.description};${entry.contextePrincipal};" +
                        "${entry.contexte2};${entry.lieA};${entry.synonyme};${entry.antonyme}"
            )
        }
    }
    try {
        FileWriter(csvFilePath).use { fileWriter ->
            fileWriter.write(csvContent)
        }
    } catch (e: IOException) {
        e.printStackTrace()
    }
}

fun importCSV(cheminFichier: String) {
    val file = File(cheminFichier).readText(Charsets.UTF_8)

    val lines = file.split("\n")
    val header = lines.first().split(";")
    val dataLines = lines.drop(1)

    dataLines.forEach { line ->
        // Check if the line is not empty or blank
        if (line.isNotBlank()) {
            val values = line.replace("\r", "").replace("\n", "").split(";")

            values.forEach { value ->
                println(value)
            }
            val mot = header.zip(values).toMap().toMutableMap()
            val nouveauMot = Mot(
                nom = mot["Mot"]!!,
                description = mot["Description"]!!,
                contextePrincipal = mot["Contexte principal"]!!,
                contexte2 = mot["Contexte 2"]!!,
                lieA = mot["Lié à"]!!,
                synonyme = mot["Synonyme"]!!,
                antonyme = mot["Antonyme"]!!
            )

            if (mot["Antonyme"] == "\r") {
                mot["Antonyme"] = ""
            }
            if (mot["Mot"] == null || mot["Mot"] == "") {
                return
            }
            if (!verifierChampsRequis(mot)) {
                return
            }
            ajouterMotAuGlossaire(nouveauMot)
        }
    }
}



fun verifierChampsRequis(mot: Map<String, String>): Boolean {
    //Vérifier que les headers sont égales aux champs requis
    val champsRequis = listOf(
        "Mot",
        "Description",
        "Contexte principal",
        "Contexte 2",
        "Lié à",
        "Synonyme",
        "Antonyme"
    )

    val motKeys = mot.keys.map { it.trim() }

    motKeys.forEach { key ->
        if (!champsRequis.contains(key)) {
            println("Le champ $key n'est pas reconnu")
            return false
        }
    }

    return true

}

@Composable
fun choixLangagePage(
    onRetourClick: () -> Unit,
    onPythonClick: () -> Unit,
    onJavaClick: () -> Unit,
    onJavaScriptClick: () -> Unit
) {

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Top, // Align content at the top
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text(text = "Choix du langage", style = MaterialTheme.typography.h3)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()  // Fills the maximum available width
            .padding(50.dp),
        // allign the content in the center vertically and horizontally
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {
        Button(
            onClick = onPythonClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )
        ) {
            Text("Python")
        }

        Button(
            onClick = onJavaClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )) {
            Text("Java")
        }

        Button(
            onClick = onJavaScriptClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )) {
            Text("JavaScript")
        }
    }

    // Bouton retour positionné tout en bas et centré horizontalement
    Column(
        modifier = Modifier.fillMaxSize().padding(20.dp),
        verticalArrangement = Arrangement.Bottom,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Button(
            onClick = onRetourClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )) {
            Text("Retour")
        }
    }

}

@Composable
fun glossairePage(
    onAjouterMotClick: () -> Unit,
    onImporterClick: () -> Unit,
    onExporterClick: () -> Unit,
    onRetourClick: () -> Unit
) {

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Top, // Align content at the top
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Text(text = "Glossaire", style = MaterialTheme.typography.h3)
    }

    Column(
        modifier = Modifier
            .fillMaxSize()  // Fills the maximum available width
            .padding(16.dp),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally

    ) {

        Button(
            onClick = onAjouterMotClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )) {
            Text("Ajouter un mot")
        }

        Button(
            onClick = onImporterClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )) {
            Text("Importer un fichier CSV")
        }

        Button(
            onClick = onExporterClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )) {
            Text("Exporter un fichier CSV")
        }

    }

    // Bouton retour positionné tout en bas et centré horizontalement
    Column(
        modifier = Modifier.fillMaxSize().padding(20.dp),
        verticalArrangement = Arrangement.Bottom,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Button(
            onClick = onRetourClick,
            colors = ButtonDefaults.buttonColors(
                backgroundColor = customRedColor,
                contentColor = Color.White
            )) {
            Text("Retour")
        }
    }

}


@Composable
fun formulairePage(onAnnulerClick: () -> Unit) {

    MaterialTheme {
        Column(
            modifier = Modifier.padding(16.dp),
            verticalArrangement = Arrangement.spacedBy(8.dp),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            Text(text = "Nouveau contexte métier", style = MaterialTheme.typography.h5)

            val nom = remember { mutableStateOf("") }
            TextField(
                value = nom.value,
                onValueChange = { nom.value = it },
                label = { Text("Mot") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor, // Change this to the color you want
                    unfocusedIndicatorColor = Color.Gray // Change this to the color you want
                )
            )

            val description = remember { mutableStateOf("") }
            TextField(
                value = description.value,
                onValueChange = { description.value = it },
                label = { Text("Description") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor, // Change this to the color you want
                    unfocusedIndicatorColor = Color.Gray // Change this to the color you want
                )
            )

            val contextePrincipal = remember { mutableStateOf("") }
            TextField(
                value = contextePrincipal.value,
                onValueChange = { contextePrincipal.value = it },
                label = { Text("Contexte principal") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor, // Change this to the color you want
                    unfocusedIndicatorColor = Color.Gray // Change this to the color you want
                )
            )

            val contexte2 = remember { mutableStateOf("") }
            TextField(
                value = contexte2.value,
                onValueChange = { contexte2.value = it },
                label = { Text("Contexte 2") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor, // Change this to the color you want
                    unfocusedIndicatorColor = Color.Gray // Change this to the color you want
                )
            )

            val lieA = remember { mutableStateOf("") }
            TextField(
                value = lieA.value,
                onValueChange = { lieA.value = it },
                label = { Text("Lié à") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor, // Change this to the color you want
                    unfocusedIndicatorColor = Color.Gray // Change this to the color you want
                )
            )

            val synonyme = remember { mutableStateOf("") }
            TextField(
                value = synonyme.value,
                onValueChange = { synonyme.value = it },
                label = { Text("Synonyme") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor, // Change this to the color you want
                    unfocusedIndicatorColor = Color.Gray // Change this to the color you want
                )
            )

            val antonyme = remember { mutableStateOf("") }
            TextField(
                value = antonyme.value,
                onValueChange = { antonyme.value = it },
                label = { Text("Antonyme") },
                colors = TextFieldDefaults.textFieldColors(
                    focusedIndicatorColor = customRedColor, // Change this to the color you want
                    unfocusedIndicatorColor = Color.Gray // Change this to the color you want
                )
            )

            Row(
                modifier = Modifier.fillMaxWidth(),
                horizontalArrangement = Arrangement.SpaceEvenly
            ) {
                Button(
                    onClick = onAnnulerClick,
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Icon(imageVector = Icons.Default.Close, contentDescription = null)
                    Text("Annuler")
                }
                Button(
                    onClick = {
                        // Validation du formulaire

                        val nouveauMot = Mot(
                            nom=nom.value,
                            description=description.value,
                            contextePrincipal=contextePrincipal.value,
                            contexte2=contexte2.value,
                            lieA=lieA.value,
                            synonyme=synonyme.value,
                            antonyme=antonyme.value
                        )

                        ajouterMotAuGlossaire(nouveauMot)


                    },
                    colors = ButtonDefaults.buttonColors(
                        backgroundColor = customRedColor,
                        contentColor = Color.White
                    )
                ) {
                    Icon(imageVector = Icons.Default.Check, contentDescription = null)
                    Text("Valider")
                }
            }

        }
    }
}

fun chargerDonneesDepuisFichier(): List<Mot> {
    //if file is empty, return empty list
    if (!File("glossaire.json").exists() || File("glossaire.json").length() == 0L) {
        return emptyList()
    }
    return try {
        val content = File("glossaire.json").readText()
        Json.decodeFromString<List<Mot>>(content)
    } catch (e: IOException) {
        e.printStackTrace()
        emptyList() // Handle the exception as needed
    }
}

fun sauvegarderDonneesDansFichier(listeMots: List<Mot>) {
    try {
        val content = Json.encodeToString(listeMots)
        File("glossaire.json").writeText(content)
    } catch (e: IOException) {
        e.printStackTrace()
        // Handle the exception as needed
    }
}

fun ajouterMotAuGlossaire(nouveauMot: Mot) {
    val listeMots = chargerDonneesDepuisFichier().toMutableList()
    listeMots.add(nouveauMot)
    sauvegarderDonneesDansFichier(listeMots)
}


fun main() = application {
    val state = rememberWindowState(
        placement = WindowPlacement.Floating,
        position = WindowPosition(Alignment.Center),
        isMinimized = false,
        width = 800.dp,
        height = 600.dp
    )

    Window(
        title = "Quali'Nomme",
        resizable = true,
        state = state,
        icon = painterResource("assets/logo/logo.png"),
        onCloseRequest = ::exitApplication
    ) {
        app()
    }
}
